// Generated by CoffeeScript 1.9.1
(function() {
  var _alphanumerify, _clone, _main, _map_to_authorizations, _map_to_model, _map_to_operation, _map_to_parameters, _map_to_responses, _markdown_to_html, _merge, _remove_falsey, init, marked;

  marked = require('marked');

  _clone = function(a) {
    var b, k, v;
    b = {};
    for (k in a) {
      v = a[k];
      b[k] = v;
    }
    return b;
  };

  _merge = function(a, b) {
    var c, k, v;
    c = _clone(a);
    for (k in b) {
      v = b[k];
      c[k] = v;
    }
    return c;
  };

  _remove_falsey = function(a) {
    var k, v;
    for (k in a) {
      v = a[k];
      if (!v) {
        delete a[k];
      }
    }
    return a;
  };

  _alphanumerify = (function(_this) {
    return function(str, replacement) {
      var re;
      if (replacement == null) {
        replacement = '_';
      }
      str = str.replace(/[^A-Za-z0-9]/g, replacement);
      re = new RegExp(replacement + "+", "g");
      str = str.replace(re, replacement);
      return str;
    };
  })(this);

  _markdown_to_html = (function(_this) {
    return function(str, strip_p) {
      if (strip_p == null) {
        strip_p = false;
      }
      if (str != null) {
        str = marked(str, {});
        str = str != null ? str.trim() : void 0;
        if (strip_p && /^<p>(.*)<\/p>$/.test(str)) {
          str = str.substring(3, str.length - 4);
        }
      }
      return str;
    };
  })(this);

  _map_to_responses = (function(_this) {
    return function(map, defaults, status_codes) {
      var code, input, ref, ref1, ref2, response, responses, v;
      if (defaults == null) {
        defaults = {};
      }
      if (status_codes == null) {
        status_codes = {};
      }
      responses = [];
      input = _remove_falsey(_merge(defaults, map));
      for (code in input) {
        v = input[code];
        response = {
          code: code
        };
        if (typeof v === 'string') {
          response.message = v;
        } else if (Array.isArray(v)) {
          response.message = v[0];
          response.responseModel = v[1];
        } else if (typeof v === 'object') {
          response = _merge(response, v);
        }
        if (typeof response.message === 'string' && ((ref = response.message) != null ? ref.length : void 0) > 0 && ((ref1 = status_codes[code]) != null ? ref1.length : void 0) > 0) {
          response.message = status_codes[code] + "; " + response.message;
        } else if (((ref2 = status_codes[code]) != null ? ref2.length : void 0) > 0) {
          response.message = status_codes[code];
        }
        response.message = _markdown_to_html(response.message, true);
        responses.push(response);
      }
      responses = responses.sort(function(a, b) {
        var ref3, ref4;
        return ((ref3 = a.code) != null ? ref3 : 0) - ((ref4 = b.code) != null ? ref4 : 0);
      });
      return responses;
    };
  })(this);

  _map_to_model = function(map) {
    var attr, i, k, len, model, prop, prop_name, ref, ref1, v, value, values;
    model = {};
    model.properties = {};
    for (prop_name in map) {
      values = map[prop_name];
      prop = {};
      if (Array.isArray(values)) {
        for (i = 0, len = values.length; i < len; i++) {
          value = values[i];
          switch (value) {
            case 'required':
              prop.required = true;
              break;
            case 'float':
            case 'double':
              prop.type = 'number';
              prop.format = value;
              break;
            case 'boolean':
            case 'string':
            case 'integer':
            case 'date':
              prop.type = value;
              break;
            case 'int':
              prop.type = 'integer';
              break;
            case 'str':
              prop.type = 'string';
              break;
            case 'datetime':
            case 'date-time':
            case 'dateTime':
              prop.type = 'string';
              prop.format = 'date-time';
              break;
            case 'int64':
            case 'int32':
            case 'long':
              prop.format = value;
              prop.type = 'integer';
              break;
            case 'array':
              prop.type = 'array';
              break;
            default:
              if (Array.isArray(value)) {
                prop.type = 'array';
                if (typeof value[0] === 'object') {
                  prop.items = value[0];
                } else {
                  prop.items = {
                    type: value[0]
                  };
                }
              } else if (typeof value === 'object') {
                for (k in value) {
                  v = value[k];
                  prop[k] = v;
                }
              } else {
                prop.description = value;
              }
          }
        }
      } else {
        for (attr in values) {
          value = values[attr];
          prop[attr] = value;
        }
      }
      prop.description = _markdown_to_html(prop.description, true);
      if (prop.required == null) {
        prop.required = false;
      }
      if ((((ref = prop.items) != null ? ref.ref : void 0) != null) && (((ref1 = prop.items) != null ? ref1['$ref'] : void 0) == null)) {
        prop.items['$ref'] = prop.items.ref;
        delete prop.items.ref;
      }
      if ((prop.ref != null) && ((prop != null ? prop['$ref'] : void 0) == null)) {
        prop['$ref'] = prop.ref;
        delete prop.ref;
      }
      model.properties[prop_name] = prop;
    }
    return model;
  };

  _map_to_operation = function(map) {
    var name, op, path, ref, value;
    op = {};
    path = Object.keys(map)[0];
    ref = map[path];
    for (name in ref) {
      value = ref[name];
      if (value != null) {
        switch (name) {
          case 'summary':
          case 'notes':
            op[name] = _markdown_to_html(value, name === 'summary');
            break;
          case 'returns':
          case 'type':
            if (Array.isArray(value)) {
              op.type = 'array';
              op.items = value[0];
              if ((op.items.ref != null) && (op.items['$ref'] == null)) {
                op.items['$ref'] = op.items.ref;
                delete op.items.ref;
              }
            } else {
              op.type = value;
            }
            break;
          case 'produces':
          case 'consumes':
            if (!Array.isArray(value)) {
              value = [value];
            }
            op[name] = value;
            break;
          case 'nickname':
          case 'deprecated':
            op[name] = value;
            break;
          case 'response':
          case 'responses':
          case 'responseMessages':
            op.responseMessages = _map_to_responses(value, typeof _standard_responses !== "undefined" && _standard_responses !== null ? _standard_responses : {}, status_codes);
            break;
          case 'parameters':
            op.parameters = _map_to_parameters(value);
            break;
          case 'authorization':
          case 'authorizations':
            if (value.oauth2 != null) {
              op.authorizations = _map_to_authorizations(value);
            }
            break;
          default:
            op[name] = value;
        }
      }
    }
    return [path, op];
  };

  _map_to_parameters = function(map) {
    var attr, i, k, len, name, param, params, v, value, values;
    params = [];
    for (name in map) {
      values = map[name];
      param = {};
      param.name = name;
      if (Array.isArray(values)) {
        for (i = 0, len = values.length; i < len; i++) {
          value = values[i];
          switch (value) {
            case 'path':
              param.paramType = 'path';
              break;
            case 'query':
            case 'querystring':
            case 'qs':
              param.paramType = 'query';
              break;
            case 'body':
              param.paramType = 'body';
              break;
            case 'required':
              param.required = true;
              break;
            case 'float':
            case 'double':
              param.type = 'number';
              param.format = value;
              break;
            case 'int':
            case 'integer':
              param.type = 'integer';
              break;
            case 'int32':
            case 'int64':
            case 'long':
              param.type = 'integer';
              param.format = value;
              break;
            case 'boolean':
            case 'string':
              param.type = value;
              break;
            case 'str':
              param.type = 'string';
              break;
            case 'date_time':
            case 'datetime':
            case 'dateTime':
              param.type = 'string';
              param.format = 'date-time';
              break;
            case 'date':
              param.type = 'string';
              param.format = 'date';
              break;
            default:
              if (typeof value === 'object' && Object.keys(value).length === 1) {
                for (k in value) {
                  v = value[k];
                  param[k] = v;
                }
              } else {
                param.defaultValue = value;
              }
          }
        }
      } else {
        for (attr in values) {
          value = values[attr];
          switch (attr) {
            case /(param-?)?type/i:
              param.paramType = value;
              break;
            case /(allow-?)?multiple/i:
              param.allowMultiple = value;
              break;
            default:
              param[attr] = value;
          }
        }
      }
      param.description = _markdown_to_html(param.description, true);
      if (param.required == null) {
        param.required = false;
      }
      if (param.format == null) {
        param.format = 'string';
      }
      params.push(param);
    }
    return params;
  };

  _map_to_authorizations = function(map) {
    var auths, description, scope, scopes, type, x;
    auths = {};
    for (type in map) {
      scopes = map[type];
      x = [];
      for (scope in scopes) {
        description = scopes[scope];
        description = _markdown_to_html(description, true);
        x.push({
          scope: scope,
          description: description
        });
      }
      auths[type] = x;
    }
    return auths;
  };

  init = function(self, options) {
    var ref;
    this.rest = {};
    this.rest.apis = [];
    this.rest.models = {};
    this.rest.swaggerVersion = '1.2';
    this.swagger_version = this.swaggerVersion = this.swaggerversion = (function(_this) {
      return function(ver) {
        return rest.swaggerVersion = "" + ver;
      };
    })(this);
    this.version = this.api_version = this.apiVersion = this.apiversion = (function(_this) {
      return function(ver) {
        if (ver == null) {
          ver = "1.0";
        }
        return rest.apiVersion = "" + ver;
      };
    })(this);
    this.base = this.basepath = this.base_path = this.basePath = (function(_this) {
      return function(path) {
        return rest.basePath = path;
      };
    })(this);
    if ((options != null ? options.strings : void 0) !== false) {
      this.path = 'path';
      this.query = 'query';
      this.body = 'body';
      this.json = 'application/json';
      this.pdf = 'application/pdf';
      this.multipart_form_data = 'multipart/form-data';
      this.multipart = 'multipart/form-data';
      this.csv = 'text/csv';
      this.html = 'text/html';
      this.text = 'text/plain';
      this.xml = 'application/xml';
      this.octet_stream = 'application/octet-stream';
      this.octet = 'application/octet-stream';
      this.binary = 'application/octet-stream';
      this.int = 'integer';
      this.integer = 'integer';
      this.long = 'long';
      this.float = 'float';
      this.double = 'double';
      this.boolean = 'boolean';
      this.date = 'date';
      this.datetime = 'date-time';
      this.dateTime = 'date-time';
      this.datetime = 'date-time';
      this.date_time = 'date-time';
      this.boolean = 'boolean';
      this.string = 'string';
      this.array = 'array';
      this.VOID = 'void';
      this.int32 = 'int32';
      this.int64 = 'int64';
      this.array = 'array';
      this.byte = 'byte';
      this.required = 'required';
      this.optional = '';
      this.ref = '$ref';
    }
    this.status_codes = (ref = options != null ? options.status_codes : void 0) != null ? ref : {
      200: 'OK',
      201: 'Created',
      204: 'No Content',
      400: 'Bad Request',
      401: 'Unauthorized',
      402: 'Payment Required',
      403: 'Forbidden',
      404: 'Not Found',
      405: 'Method Not Allowed',
      406: 'Not Acceptable',
      409: 'Conflict',
      412: 'Precondition Failed',
      415: 'Unsupported Media Type',
      420: 'Enhance Your Calm',
      422: 'Unprocessable Entity',
      429: 'Too Many Requests',
      500: 'Internal Server Error',
      502: 'Gateway Error',
      503: 'Service Unavailable'
    };
    this.standard_responses = this.standardresponses = (function(_this) {
      return function(map) {
        return _this._standard_responses = map;
      };
    })(this);
    this._add_model = function(name, map) {
      map = _map_to_model(map);
      if (map.id == null) {
        map.id = name;
      }
      return rest.models[name] = map;
    };
    this.MODEL = function(map) {
      var name, props, results;
      results = [];
      for (name in map) {
        props = map[name];
        results.push(_add_model(name, props));
      }
      return results;
    };
    this._add_operation = function(path, op) {
      return rest.apis.push({
        path: path,
        operations: [op]
      });
    };
    this.GET = function(map) {
      return api_method('GET', map);
    };
    this.POST = function(map) {
      return api_method('POST', map);
    };
    this.PUT = function(map) {
      return api_method('PUT', map);
    };
    this.DELETE = function(map) {
      return api_method('DELETE', map);
    };
    this.PATCH = function(map) {
      return api_method('PATCH', map);
    };
    this.HEAD = function(map) {
      return api_method('HEAD', map);
    };
    this.OPTIONS = function(map) {
      return api_method('OPTIONS', map);
    };
    this.api_method = function(method, map) {
      var op, path, ref1;
      ref1 = _map_to_operation(map), path = ref1[0], op = ref1[1];
      op.method = method;
      if (op.nickname == null) {
        op.nickname = _alphanumerify((method.toLowerCase()) + "-" + path);
      }
      return _add_operation(path, op);
    };
    this.to_json = function(spaces) {
      if (spaces == null) {
        spaces = 2;
      }
      return JSON.stringify(rest, null, spaces);
    };
    return this;
  };

  _main = (function(_this) {
    return function(argv, logfn, errfn, callback) {
      var CoffeeScript, code, data, file, fs, i, json, len, matches, optimist, options, original_argv, ref, ref1, regexp, string;
      if ((errfn != null) && (callback == null)) {
        callback = errfn;
        errfn = null;
      }
      if ((logfn != null) && (callback == null)) {
        callback = lognf;
        logfn = null;
      }
      if (argv == null) {
        argv = typeof process !== "undefined" && process !== null ? process.argv : void 0;
      }
      if (logfn == null) {
        logfn = console.log;
      }
      if (errfn == null) {
        errfn = console.error;
      }
      if (callback == null) {
        callback = (ref = typeof process !== "undefined" && process !== null ? process.exit : void 0) != null ? ref : (function() {});
      }
      original_argv = process.argv;
      process.argv = argv;
      try {
        CoffeeScript = require('coffee-script');
        optimist = require('optimist');
        fs = require('fs');
        options = {
          h: {
            alias: 'help',
            boolean: true,
            describe: "Show help"
          },
          t: {
            alias: 'indent',
            describe: "Number of spaces per indent level. Use 0 to print on one line",
            "default": 2
          },
          o: {
            alias: 'out',
            describe: "Name of file to write to, use '-'for STDOUT.",
            "default": '-'
          },
          x: {
            alias: 'suffix',
            describe: "Extension to add to ouput files. Overrides --out.",
            "default": null
          },
          r: {
            alias: 'rename',
            describe: "Regexp/String pair describing mapping between input and output filenames. Overrides --out and --suffix.",
            "default": null
          }
        };
        argv = optimist.usage('Usage: $0 ... [FILE(S)]', options).argv;
        if (argv.help) {
          optimist.showHelp(errfn);
          return callback();
        } else {
          ref1 = argv._;
          for (i = 0, len = ref1.length; i < len; i++) {
            file = ref1[i];
            data = fs.readFileSync(file);
            code = "init(this)\n" + data + "\nreturn to_json(" + argv.i + ")\n";
            json = eval(CoffeeScript.compile(code));
            if (argv.r != null) {
              matches = argv.r.match(/^(\/.+\/),((".+\")|(\'.+\'))$/);
              if ((matches != null ? matches[2] : void 0) == null) {
                errfn("Error parsing rename pair " + argv.r);
                callback(1);
                return;
              } else {
                regexp = eval(matches[1]);
                string = eval(matches[2]);
                argv.o = file.replace(regexp, string);
              }
            } else if (argv.x != null) {
              argv.o = file.replace(/^(.+)($)/, "$1." + argv.x);
            }
            if (argv.o === '-') {
              logfn(json);
            } else {
              fs.writeFileSync(argv.o, json);
            }
          }
          return callback();
        }
      } finally {
        process.argv = original_argv;
      }
    };
  })(this);

  module.exports = init;

  if (require.main === module) {
    _main();
  }

}).call(this);
